@@ -56,6 +56,7 @@ type embedDummyResourceImplementer struct {
 	pluginConfig            PluginConfig
 	upstreamServiceRelation UpstreamServiceRelation
 	pluginMetadata          PluginMetadata
+	validator               APISIXSchemaValidator
 }
 
 type dummyRoute struct{}
@@ -68,7 +69,7 @@ func (f *dummyRoute) List(_ context.Context) ([]*v1.Route, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyRoute) Create(_ context.Context, _ *v1.Route) (*v1.Route, error) {
+func (f *dummyRoute) Create(_ context.Context, _ *v1.Route, shouldCompare bool) (*v1.Route, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -76,7 +77,7 @@ func (f *dummyRoute) Delete(_ context.Context, _ *v1.Route) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummyRoute) Update(_ context.Context, _ *v1.Route) (*v1.Route, error) {
+func (f *dummyRoute) Update(_ context.Context, _ *v1.Route, shouldCompare bool) (*v1.Route, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -90,7 +91,7 @@ func (f *dummySSL) List(_ context.Context) ([]*v1.Ssl, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummySSL) Create(_ context.Context, _ *v1.Ssl) (*v1.Ssl, error) {
+func (f *dummySSL) Create(_ context.Context, _ *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -98,7 +99,7 @@ func (f *dummySSL) Delete(_ context.Context, _ *v1.Ssl) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummySSL) Update(_ context.Context, _ *v1.Ssl) (*v1.Ssl, error) {
+func (f *dummySSL) Update(_ context.Context, _ *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -112,7 +113,7 @@ func (f *dummyUpstream) List(_ context.Context) ([]*v1.Upstream, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyUpstream) Create(_ context.Context, _ *v1.Upstream) (*v1.Upstream, error) {
+func (f *dummyUpstream) Create(_ context.Context, _ *v1.Upstream, shouldCompare bool) (*v1.Upstream, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -120,7 +121,7 @@ func (f *dummyUpstream) Delete(_ context.Context, _ *v1.Upstream) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummyUpstream) Update(_ context.Context, _ *v1.Upstream) (*v1.Upstream, error) {
+func (f *dummyUpstream) Update(_ context.Context, _ *v1.Upstream, shouldCompare bool) (*v1.Upstream, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -134,7 +135,7 @@ func (f *dummyStreamRoute) List(_ context.Context) ([]*v1.StreamRoute, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyStreamRoute) Create(_ context.Context, _ *v1.StreamRoute) (*v1.StreamRoute, error) {
+func (f *dummyStreamRoute) Create(_ context.Context, _ *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -142,7 +143,7 @@ func (f *dummyStreamRoute) Delete(_ context.Context, _ *v1.StreamRoute) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummyStreamRoute) Update(_ context.Context, _ *v1.StreamRoute) (*v1.StreamRoute, error) {
+func (f *dummyStreamRoute) Update(_ context.Context, _ *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -156,7 +157,7 @@ func (f *dummyGlobalRule) List(_ context.Context) ([]*v1.GlobalRule, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyGlobalRule) Create(_ context.Context, _ *v1.GlobalRule) (*v1.GlobalRule, error) {
+func (f *dummyGlobalRule) Create(_ context.Context, _ *v1.GlobalRule, shouldCompare bool) (*v1.GlobalRule, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -164,7 +165,7 @@ func (f *dummyGlobalRule) Delete(_ context.Context, _ *v1.GlobalRule) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummyGlobalRule) Update(_ context.Context, _ *v1.GlobalRule) (*v1.GlobalRule, error) {
+func (f *dummyGlobalRule) Update(_ context.Context, _ *v1.GlobalRule, shouldCompare bool) (*v1.GlobalRule, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -178,7 +179,7 @@ func (f *dummyConsumer) List(_ context.Context) ([]*v1.Consumer, error) {
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyConsumer) Create(_ context.Context, _ *v1.Consumer) (*v1.Consumer, error) {
+func (f *dummyConsumer) Create(_ context.Context, _ *v1.Consumer, shouldCompare bool) (*v1.Consumer, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -186,7 +187,7 @@ func (f *dummyConsumer) Delete(_ context.Context, _ *v1.Consumer) error {
 	return ErrClusterNotExist
 }
 
-func (f *dummyConsumer) Update(_ context.Context, _ *v1.Consumer) (*v1.Consumer, error) {
+func (f *dummyConsumer) Update(_ context.Context, _ *v1.Consumer, shouldCompare bool) (*v1.Consumer, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -232,7 +233,7 @@ func (f *dummyPluginConfig) List(_ context.Context) ([]*v1.PluginConfig, error)
 	return nil, ErrClusterNotExist
 }
 
-func (f *dummyPluginConfig) Create(_ context.Context, _ *v1.PluginConfig) (*v1.PluginConfig, error) {
+func (f *dummyPluginConfig) Create(_ context.Context, _ *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -240,7 +241,7 @@ func (f *dummyPluginConfig) Delete(_ context.Context, _ *v1.PluginConfig) error
 	return ErrClusterNotExist
 }
 
-func (f *dummyPluginConfig) Update(_ context.Context, _ *v1.PluginConfig) (*v1.PluginConfig, error) {
+func (f *dummyPluginConfig) Update(_ context.Context, _ *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
 	return nil, ErrClusterNotExist
 }
 
@@ -273,9 +274,26 @@ func (f *dummyPluginMetadata) List(_ context.Context) ([]*v1.PluginMetadata, err
 func (f *dummyPluginMetadata) Delete(_ context.Context, _ *v1.PluginMetadata) error {
 	return ErrClusterNotExist
 }
-func (f *dummyPluginMetadata) Update(_ context.Context, _ *v1.PluginMetadata) (*v1.PluginMetadata, error) {
+func (f *dummyPluginMetadata) Update(_ context.Context, _ *v1.PluginMetadata, shouldCompare bool) (*v1.PluginMetadata, error) {
 	return nil, ErrClusterNotExist
 }
+func (f *dummyPluginMetadata) Create(_ context.Context, _ *v1.PluginMetadata, shouldCompare bool) (*v1.PluginMetadata, error) {
+	return nil, ErrClusterNotExist
+}
+
+type dummyValidator struct{}
+
+func newDummyValidator() APISIXSchemaValidator {
+	return &dummyValidator{}
+}
+
+func (d *dummyValidator) ValidateHTTPPluginSchema(plugins v1.Plugins) (bool, error) {
+	return true, nil
+}
+
+func (d *dummyValidator) ValidateStreamPluginSchema(plugins v1.Plugins) (bool, error) {
+	return true, nil
+}
 
 func (nc *nonExistentCluster) Route() Route {
 	return nc.route
@@ -305,6 +323,10 @@ func (nc *nonExistentCluster) Plugin() Plugin {
 	return nc.plugin
 }
 
+func (nc *nonExistentCluster) Validator() APISIXSchemaValidator {
+	return nc.validator
+}
+
 func (nc *nonExistentCluster) PluginConfig() PluginConfig {
 	return nc.pluginConfig
 }
