@@ -18,6 +18,7 @@ package apisix
 import (
 	"context"
 	"encoding/json"
+	"fmt"
 
 	"go.uber.org/zap"
 
@@ -66,35 +67,8 @@ func (pc *pluginConfigClient) Get(ctx context.Context, name string) (*v1.PluginC
 	}
 
 	// TODO Add mutex here to avoid dog-pile effect.
-	url := pc.url + "/" + rid
-	resp, err := pc.cluster.getResource(ctx, url, "pluginConfig")
-	pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
+	pluginConfig, err = pc.cluster.GetPluginConfig(ctx, pc.url, rid)
 	if err != nil {
-		if err == cache.ErrNotFound {
-			log.Warnw("pluginConfig not found",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", pc.cluster.name),
-			)
-		} else {
-			log.Errorw("failed to get pluginConfig from APISIX",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", pc.cluster.name),
-				zap.Error(err),
-			)
-		}
-		return nil, err
-	}
-
-	pluginConfig, err = resp.pluginConfig()
-	if err != nil {
-		log.Errorw("failed to convert pluginConfig item",
-			zap.String("url", pc.url),
-			zap.String("pluginConfig_key", resp.Key),
-			zap.String("pluginConfig_value", string(resp.Value)),
-			zap.Error(err),
-		)
 		return nil, err
 	}
 
@@ -113,7 +87,6 @@ func (pc *pluginConfigClient) List(ctx context.Context) ([]*v1.PluginConfig, err
 		zap.String("url", pc.url),
 	)
 	pluginConfigItems, err := pc.cluster.listResource(ctx, pc.url, "pluginConfig")
-	pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
 	if err != nil {
 		log.Errorf("failed to list pluginConfig: %s", err)
 		return nil, err
@@ -139,7 +112,11 @@ func (pc *pluginConfigClient) List(ctx context.Context) ([]*v1.PluginConfig, err
 	return items, nil
 }
 
-func (pc *pluginConfigClient) Create(ctx context.Context, obj *v1.PluginConfig) (*v1.PluginConfig, error) {
+func (pc *pluginConfigClient) Create(ctx context.Context, obj *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
+	if v, skip := skipRequest(pc.cluster, shouldCompare, pc.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to create pluginConfig",
 		zap.String("name", obj.Name),
 		zap.Any("plugins", obj.Plugins),
@@ -158,7 +135,6 @@ func (pc *pluginConfigClient) Create(ctx context.Context, obj *v1.PluginConfig)
 	url := pc.url + "/" + obj.ID
 	log.Debugw("creating pluginConfig", zap.ByteString("body", data), zap.String("url", url))
 	resp, err := pc.cluster.createResource(ctx, url, "pluginConfig", data)
-	pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
 	if err != nil {
 		log.Errorf("failed to create pluginConfig: %s", err)
 		return nil, err
@@ -172,6 +148,10 @@ func (pc *pluginConfigClient) Create(ctx context.Context, obj *v1.PluginConfig)
 		log.Errorf("failed to reflect pluginConfig create to cache: %s", err)
 		return nil, err
 	}
+	if err := pc.cluster.generatedObjCache.InsertPluginConfig(obj); err != nil {
+		log.Errorf("failed to cache generated pluginConfig object: %s", err)
+		return nil, err
+	}
 	return pluginConfig, nil
 }
 
@@ -186,22 +166,30 @@ func (pc *pluginConfigClient) Delete(ctx context.Context, obj *v1.PluginConfig)
 	if err := pc.cluster.HasSynced(ctx); err != nil {
 		return err
 	}
+	url := pc.url + "/" + obj.ID
+	if err := pc.cluster.deleteResource(ctx, url, "pluginConfig"); err != nil {
+		return err
+	}
 	if err := pc.cluster.cache.DeletePluginConfig(obj); err != nil {
 		log.Errorf("failed to reflect pluginConfig delete to cache: %s", err)
 		if err != cache.ErrNotFound {
 			return err
 		}
 	}
-	url := pc.url + "/" + obj.ID
-	if err := pc.cluster.deleteResource(ctx, url, "pluginConfig"); err != nil {
-		pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
-		return err
+	if err := pc.cluster.generatedObjCache.DeletePluginConfig(obj); err != nil {
+		log.Errorf("failed to reflect pluginConfig delete to generated cache: %s", err)
+		if err != cache.ErrNotFound {
+			return err
+		}
 	}
-	pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
 	return nil
 }
 
-func (pc *pluginConfigClient) Update(ctx context.Context, obj *v1.PluginConfig) (*v1.PluginConfig, error) {
+func (pc *pluginConfigClient) Update(ctx context.Context, obj *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
+	if v, skip := skipRequest(pc.cluster, shouldCompare, pc.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to update pluginConfig",
 		zap.String("id", obj.ID),
 		zap.String("name", obj.Name),
@@ -218,7 +206,6 @@ func (pc *pluginConfigClient) Update(ctx context.Context, obj *v1.PluginConfig)
 	}
 	url := pc.url + "/" + obj.ID
 	resp, err := pc.cluster.updateResource(ctx, url, "pluginConfig", body)
-	pc.cluster.metricsCollector.IncrAPISIXRequest("pluginConfig")
 	if err != nil {
 		return nil, err
 	}
@@ -230,5 +217,125 @@ func (pc *pluginConfigClient) Update(ctx context.Context, obj *v1.PluginConfig)
 		log.Errorf("failed to reflect pluginConfig update to cache: %s", err)
 		return nil, err
 	}
+	if err := pc.cluster.generatedObjCache.InsertPluginConfig(obj); err != nil {
+		log.Errorf("failed to cache generated pluginConfig object: %s", err)
+		return nil, err
+	}
 	return pluginConfig, nil
 }
+
+type pluginConfigMem struct {
+	url string
+
+	resource string
+	cluster  *cluster
+}
+
+func newPluginConfigMem(c *cluster) PluginConfig {
+	return &pluginConfigMem{
+		url:      c.baseURL + "/plugin_configs",
+		resource: "plugin_configs",
+		cluster:  c,
+	}
+}
+
+func (r *pluginConfigMem) Get(ctx context.Context, name string) (*v1.PluginConfig, error) {
+	log.Debugw("try to look up pluginConfig",
+		zap.String("name", name),
+		zap.String("url", r.url),
+		zap.String("cluster", r.cluster.name),
+	)
+	rid := id.GenID(name)
+	pluginConfig, err := r.cluster.cache.GetPluginConfig(rid)
+	if err != nil {
+		log.Errorw("failed to find pluginConfig in cache, will try to lookup from APISIX",
+			zap.String("name", name),
+			zap.Error(err),
+		)
+	}
+	return pluginConfig, nil
+}
+
+// List is only used in cache warming up. So here just pass through
+// to APISIX.
+func (r *pluginConfigMem) List(ctx context.Context) ([]*v1.PluginConfig, error) {
+	log.Debugw("try to list resource in APISIX",
+		zap.String("cluster", r.cluster.name),
+		zap.String("resource", r.resource),
+	)
+	pluginConfigItems, err := r.cluster.cache.ListPluginConfigs()
+	if err != nil {
+		log.Errorf("failed to list %s: %s", r.resource, err)
+		return nil, err
+	}
+	return pluginConfigItems, nil
+}
+
+func (r *pluginConfigMem) Create(ctx context.Context, obj *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	if ok, err := r.cluster.validator.ValidateHTTPPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.CreateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertPluginConfig(obj); err != nil {
+		log.Errorf("failed to reflect plugin_config create to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
+
+func (r *pluginConfigMem) Delete(ctx context.Context, obj *v1.PluginConfig) error {
+	if ok, err := r.deleteCheck(ctx, obj); !ok {
+		log.Debug("failed to delete upstream", zap.Error(err))
+		return cache.ErrStillInUse
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return err
+	}
+	r.cluster.DeleteResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.DeletePluginConfig(obj); err != nil {
+		log.Errorf("failed to reflect plugin_config delete to cache: %s", err)
+		return err
+	}
+	return nil
+}
+
+func (r *pluginConfigMem) Update(ctx context.Context, obj *v1.PluginConfig, shouldCompare bool) (*v1.PluginConfig, error) {
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	if ok, err := r.cluster.validator.ValidateHTTPPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.UpdateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertPluginConfig(obj); err != nil {
+		log.Errorf("failed to reflect plugin_config update to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
+
+// TODO: Maintain a reference count for each object without having to poll each time
+func (u *pluginConfigMem) deleteCheck(ctx context.Context, obj *v1.PluginConfig) (bool, error) {
+	routes, _ := u.cluster.route.List(ctx)
+	if routes == nil {
+		return true, nil
+	}
+	for _, route := range routes {
+		if route.PluginConfigId == obj.ID {
+			return false, fmt.Errorf("can not delete this plugin_config, route.id=%s is still using it now", route.ID)
+		}
+	}
+	return true, nil
+}
