@@ -65,36 +65,9 @@ func (r *routeClient) Get(ctx context.Context, name string) (*v1.Route, error) {
 		)
 	}
 
-	// TODO Add mutex here to avoid dog-pile effection.
-	url := r.url + "/" + rid
-	resp, err := r.cluster.getResource(ctx, url, "route")
-	r.cluster.metricsCollector.IncrAPISIXRequest("route")
+	// TODO Add mutex here to avoid dog-pile effect
+	route, err = r.cluster.GetRoute(ctx, r.url, rid)
 	if err != nil {
-		if err == cache.ErrNotFound {
-			log.Warnw("route not found",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", r.cluster.name),
-			)
-		} else {
-			log.Errorw("failed to get route from APISIX",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", r.cluster.name),
-				zap.Error(err),
-			)
-		}
-		return nil, err
-	}
-
-	route, err = resp.route()
-	if err != nil {
-		log.Errorw("failed to convert route item",
-			zap.String("url", r.url),
-			zap.String("route_key", resp.Key),
-			zap.String("route_value", string(resp.Value)),
-			zap.Error(err),
-		)
 		return nil, err
 	}
 
@@ -113,7 +86,6 @@ func (r *routeClient) List(ctx context.Context) ([]*v1.Route, error) {
 		zap.String("url", r.url),
 	)
 	routeItems, err := r.cluster.listResource(ctx, r.url, "route")
-	r.cluster.metricsCollector.IncrAPISIXRequest("route")
 	if err != nil {
 		log.Errorf("failed to list routes: %s", err)
 		return nil, err
@@ -139,7 +111,11 @@ func (r *routeClient) List(ctx context.Context) ([]*v1.Route, error) {
 	return items, nil
 }
 
-func (r *routeClient) Create(ctx context.Context, obj *v1.Route) (*v1.Route, error) {
+func (r *routeClient) Create(ctx context.Context, obj *v1.Route, shouldCompare bool) (*v1.Route, error) {
+	if v, skip := skipRequest(r.cluster, shouldCompare, r.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to create route",
 		zap.Strings("hosts", obj.Hosts),
 		zap.String("name", obj.Name),
@@ -158,7 +134,6 @@ func (r *routeClient) Create(ctx context.Context, obj *v1.Route) (*v1.Route, err
 	url := r.url + "/" + obj.ID
 	log.Debugw("creating route", zap.ByteString("body", data), zap.String("url", url))
 	resp, err := r.cluster.createResource(ctx, url, "route", data)
-	r.cluster.metricsCollector.IncrAPISIXRequest("route")
 	if err != nil {
 		log.Errorf("failed to create route: %s", err)
 		return nil, err
@@ -172,6 +147,10 @@ func (r *routeClient) Create(ctx context.Context, obj *v1.Route) (*v1.Route, err
 		log.Errorf("failed to reflect route create to cache: %s", err)
 		return nil, err
 	}
+	if err := r.cluster.generatedObjCache.InsertRoute(obj); err != nil {
+		log.Errorf("failed to cache generated route object: %s", err)
+		return nil, err
+	}
 	return route, nil
 }
 
@@ -187,20 +166,28 @@ func (r *routeClient) Delete(ctx context.Context, obj *v1.Route) error {
 	}
 	url := r.url + "/" + obj.ID
 	if err := r.cluster.deleteResource(ctx, url, "route"); err != nil {
-		r.cluster.metricsCollector.IncrAPISIXRequest("route")
 		return err
 	}
-	r.cluster.metricsCollector.IncrAPISIXRequest("route")
 	if err := r.cluster.cache.DeleteRoute(obj); err != nil {
 		log.Errorf("failed to reflect route delete to cache: %s", err)
 		if err != cache.ErrNotFound {
 			return err
 		}
 	}
+	if err := r.cluster.generatedObjCache.DeleteRoute(obj); err != nil {
+		log.Errorf("failed to reflect route delete to generated cache: %s", err)
+		if err != cache.ErrNotFound {
+			return err
+		}
+	}
 	return nil
 }
 
-func (r *routeClient) Update(ctx context.Context, obj *v1.Route) (*v1.Route, error) {
+func (r *routeClient) Update(ctx context.Context, obj *v1.Route, shouldCompare bool) (*v1.Route, error) {
+	if v, skip := skipRequest(r.cluster, shouldCompare, r.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to update route",
 		zap.String("id", obj.ID),
 		zap.String("name", obj.Name),
@@ -216,7 +203,6 @@ func (r *routeClient) Update(ctx context.Context, obj *v1.Route) (*v1.Route, err
 	}
 	url := r.url + "/" + obj.ID
 	resp, err := r.cluster.updateResource(ctx, url, "route", body)
-	r.cluster.metricsCollector.IncrAPISIXRequest("route")
 	if err != nil {
 		return nil, err
 	}
@@ -228,5 +214,107 @@ func (r *routeClient) Update(ctx context.Context, obj *v1.Route) (*v1.Route, err
 		log.Errorf("failed to reflect route update to cache: %s", err)
 		return nil, err
 	}
+	if err := r.cluster.generatedObjCache.InsertRoute(obj); err != nil {
+		log.Errorf("failed to cache generated route object: %s", err)
+		return nil, err
+	}
 	return route, nil
 }
+
+type routeMem struct {
+	url string
+
+	resource string
+	cluster  *cluster
+}
+
+func newRouteMem(c *cluster) Route {
+	return &routeMem{
+		url:      c.baseURL + "/routes",
+		resource: "routes",
+		cluster:  c,
+	}
+}
+
+func (r *routeMem) Get(ctx context.Context, name string) (*v1.Route, error) {
+	log.Debugw("try to look up route",
+		zap.String("name", name),
+		zap.String("cluster", r.cluster.name),
+	)
+	rid := id.GenID(name)
+	route, err := r.cluster.cache.GetRoute(rid)
+	if err != nil {
+		log.Errorw("failed to find route in cache",
+			zap.String("name", name),
+			zap.Error(err),
+		)
+		return nil, err
+	}
+	return route, nil
+}
+
+// List is only used in cache warming up. So here just pass through
+// to APISIX.
+func (r *routeMem) List(ctx context.Context) ([]*v1.Route, error) {
+	log.Debugw("try to list resource in APISIX",
+		zap.String("cluster", r.cluster.name),
+		zap.String("resource", r.resource),
+	)
+	routes, err := r.cluster.cache.ListRoutes()
+	if err != nil {
+		log.Errorf("failed to list %s: %s", r.resource, err)
+		return nil, err
+	}
+	return routes, nil
+}
+
+func (r *routeMem) Create(ctx context.Context, obj *v1.Route, shouldCompare bool) (*v1.Route, error) {
+	if ok, err := r.cluster.validator.ValidateHTTPPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.CreateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertRoute(obj); err != nil {
+		log.Errorf("failed to reflect route create to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
+
+func (r *routeMem) Delete(ctx context.Context, obj *v1.Route) error {
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return err
+	}
+	r.cluster.DeleteResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.DeleteRoute(obj); err != nil {
+		log.Errorf("failed to reflect route delete to cache: %s", err)
+		return nil
+	}
+	return nil
+}
+
+func (r *routeMem) Update(ctx context.Context, obj *v1.Route, shouldCompare bool) (*v1.Route, error) {
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	if ok, err := r.cluster.validator.ValidateHTTPPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.UpdateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertRoute(obj); err != nil {
+		log.Errorf("failed to reflect route update to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
