@@ -46,6 +46,7 @@ func newSSLClient(c *cluster) SSL {
 	}
 }
 
+// name is namespace_sslname
 func (s *sslClient) Get(ctx context.Context, name string) (*v1.Ssl, error) {
 	log.Debugw("try to look up ssl",
 		zap.String("name", name),
@@ -69,33 +70,9 @@ func (s *sslClient) Get(ctx context.Context, name string) (*v1.Ssl, error) {
 		)
 	}
 
-	// TODO Add mutex here to avoid dog-pile effection.
-	url := s.url + "/" + sid
-	resp, err := s.cluster.getResource(ctx, url, "ssl")
+	// TODO Add mutex here to avoid dog-pile effect.
+	ssl, err = s.cluster.GetSSL(ctx, s.url, sid)
 	if err != nil {
-		if err == cache.ErrNotFound {
-			log.Warnw("ssl not found",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", s.cluster.name),
-			)
-		} else {
-			log.Errorw("failed to get ssl from APISIX",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", s.cluster.name),
-				zap.Error(err),
-			)
-		}
-		return nil, err
-	}
-	ssl, err = resp.ssl()
-	if err != nil {
-		log.Errorw("failed to convert ssl item",
-			zap.String("url", s.url),
-			zap.String("ssl_key", resp.Key),
-			zap.Error(err),
-		)
 		return nil, err
 	}
 
@@ -115,7 +92,6 @@ func (s *sslClient) List(ctx context.Context) ([]*v1.Ssl, error) {
 	)
 
 	sslItems, err := s.cluster.listResource(ctx, s.url, "ssl")
-	s.cluster.metricsCollector.IncrAPISIXRequest("ssl")
 	if err != nil {
 		log.Errorf("failed to list ssl: %s", err)
 		return nil, err
@@ -139,7 +115,11 @@ func (s *sslClient) List(ctx context.Context) ([]*v1.Ssl, error) {
 	return items, nil
 }
 
-func (s *sslClient) Create(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
+func (s *sslClient) Create(ctx context.Context, obj *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
+	if v, skip := skipRequest(s.cluster, shouldCompare, s.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to create ssl",
 		zap.String("cluster", s.cluster.name),
 		zap.String("url", s.url),
@@ -155,7 +135,6 @@ func (s *sslClient) Create(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
 	url := s.url + "/" + obj.ID
 	log.Debugw("creating ssl", zap.ByteString("body", data), zap.String("url", url))
 	resp, err := s.cluster.createResource(ctx, url, "ssl", data)
-	s.cluster.metricsCollector.IncrAPISIXRequest("ssl")
 	if err != nil {
 		log.Errorf("failed to create ssl: %s", err)
 		return nil, err
@@ -169,6 +148,10 @@ func (s *sslClient) Create(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
 		log.Errorf("failed to reflect ssl create to cache: %s", err)
 		return nil, err
 	}
+	if err := s.cluster.generatedObjCache.InsertSSL(obj); err != nil {
+		log.Errorf("failed to reflect generated ssl create to cache: %s", err)
+		return nil, err
+	}
 	return ssl, nil
 }
 
@@ -183,20 +166,28 @@ func (s *sslClient) Delete(ctx context.Context, obj *v1.Ssl) error {
 	}
 	url := s.url + "/" + obj.ID
 	if err := s.cluster.deleteResource(ctx, url, "ssl"); err != nil {
-		s.cluster.metricsCollector.IncrAPISIXRequest("ssl")
 		return err
 	}
-	s.cluster.metricsCollector.IncrAPISIXRequest("ssl")
 	if err := s.cluster.cache.DeleteSSL(obj); err != nil {
 		log.Errorf("failed to reflect ssl delete to cache: %s", err)
 		if err != cache.ErrNotFound {
 			return err
 		}
 	}
+	if err := s.cluster.generatedObjCache.DeleteSSL(obj); err != nil {
+		log.Errorf("failed to reflect ssl delete to generated cache: %s", err)
+		if err != cache.ErrNotFound {
+			return err
+		}
+	}
 	return nil
 }
 
-func (s *sslClient) Update(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
+func (s *sslClient) Update(ctx context.Context, obj *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
+	if v, skip := skipRequest(s.cluster, shouldCompare, s.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to update ssl",
 		zap.String("id", obj.ID),
 		zap.String("cluster", s.cluster.name),
@@ -211,7 +202,6 @@ func (s *sslClient) Update(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
 		return nil, err
 	}
 	resp, err := s.cluster.updateResource(ctx, url, "ssl", data)
-	s.cluster.metricsCollector.IncrAPISIXRequest("ssl")
 	if err != nil {
 		return nil, err
 	}
@@ -223,5 +213,107 @@ func (s *sslClient) Update(ctx context.Context, obj *v1.Ssl) (*v1.Ssl, error) {
 		log.Errorf("failed to reflect ssl update to cache: %s", err)
 		return nil, err
 	}
+	if err := s.cluster.generatedObjCache.InsertSSL(obj); err != nil {
+		log.Errorf("failed to reflect generated ssl update to cache: %s", err)
+		return nil, err
+	}
 	return ssl, nil
 }
+
+type sslMem struct {
+	url string
+
+	resource string
+	cluster  *cluster
+
+	keyEncryptSalt string
+}
+
+func newSSLMem(c *cluster) SSL {
+	return &sslMem{
+		url:            c.baseURL + "/ssls",
+		resource:       "ssls",
+		cluster:        c,
+		keyEncryptSalt: c.sslKeyEncryptSalt,
+	}
+}
+
+func (r *sslMem) Get(ctx context.Context, name string) (*v1.Ssl, error) {
+	log.Debugw("try to look up ssl",
+		zap.String("name", name),
+		zap.String("cluster", r.cluster.name),
+	)
+	rid := id.GenID(name)
+	ssl, err := r.cluster.cache.GetSSL(rid)
+	if err != nil {
+		log.Errorw("failed to find ssl in cache, will try to lookup from APISIX",
+			zap.String("name", name),
+			zap.Error(err),
+		)
+		return nil, err
+	}
+	return ssl, nil
+}
+
+// List is only used in cache warming up. So here just pass through
+// to APISIX.
+func (r *sslMem) List(ctx context.Context) ([]*v1.Ssl, error) {
+	log.Debugw("try to list resource in APISIX",
+		zap.String("cluster", r.cluster.name),
+		zap.String("resource", r.resource),
+	)
+	ssls, err := r.cluster.cache.ListSSL()
+	if err != nil {
+		log.Errorf("failed to list %s: %s", r.resource, err)
+		return nil, err
+	}
+	return ssls, nil
+}
+
+func (r *sslMem) Create(ctx context.Context, obj *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
+	if ssl, _ := r.cluster.cache.GetSSL(obj.ID); ssl != nil {
+		return r.Update(ctx, obj, shouldCompare)
+	}
+	pkey, err := AesEencryptPrivatekey([]byte(obj.Key), []byte(r.keyEncryptSalt))
+	if err != nil {
+		return nil, err
+	}
+	obj.Key = pkey
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.CreateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertSSL(obj); err != nil {
+		log.Errorf("failed to reflect ssl create to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
+
+func (r *sslMem) Delete(ctx context.Context, obj *v1.Ssl) error {
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return err
+	}
+	r.cluster.DeleteResource(r.resource, obj.ID, data)
+	return nil
+}
+
+func (r *sslMem) Update(ctx context.Context, obj *v1.Ssl, shouldCompare bool) (*v1.Ssl, error) {
+	pkey, err := AesEencryptPrivatekey([]byte(obj.Key), []byte(r.keyEncryptSalt))
+	if err != nil {
+		return nil, err
+	}
+	obj.Key = pkey
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.UpdateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertSSL(obj); err != nil {
+		log.Errorf("failed to reflect ssl update to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
