@@ -70,35 +70,9 @@ func (r *streamRouteClient) Get(ctx context.Context, name string) (*v1.StreamRou
 		)
 	}
 
-	// TODO Add mutex here to avoid dog-pile effection.
-	url := r.url + "/" + rid
-	resp, err := r.cluster.getResource(ctx, url, "streamRoute")
+	// TODO Add mutex here to avoid dog-pile effect.
+	streamRoute, err = r.cluster.GetStreamRoute(ctx, r.url, rid)
 	if err != nil {
-		if err == cache.ErrNotFound {
-			log.Warnw("stream_route not found",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", r.cluster.name),
-			)
-		} else {
-			log.Errorw("failed to get stream_route from APISIX",
-				zap.String("name", name),
-				zap.String("url", url),
-				zap.String("cluster", r.cluster.name),
-				zap.Error(err),
-			)
-		}
-		return nil, err
-	}
-
-	streamRoute, err = resp.streamRoute()
-	if err != nil {
-		log.Errorw("failed to convert stream_route item",
-			zap.String("url", r.url),
-			zap.String("stream_route_key", resp.Key),
-			zap.String("stream_route_value", string(resp.Value)),
-			zap.Error(err),
-		)
 		return nil, err
 	}
 
@@ -117,7 +91,6 @@ func (r *streamRouteClient) List(ctx context.Context) ([]*v1.StreamRoute, error)
 		zap.String("url", r.url),
 	)
 	streamRouteItems, err := r.cluster.listResource(ctx, r.url, "streamRoute")
-	r.cluster.metricsCollector.IncrAPISIXRequest("streamRoute")
 	if err != nil {
 		log.Errorf("failed to list stream_routes: %s", err)
 		return nil, err
@@ -142,7 +115,11 @@ func (r *streamRouteClient) List(ctx context.Context) ([]*v1.StreamRoute, error)
 	return items, nil
 }
 
-func (r *streamRouteClient) Create(ctx context.Context, obj *v1.StreamRoute) (*v1.StreamRoute, error) {
+func (r *streamRouteClient) Create(ctx context.Context, obj *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
+	if v, skip := skipRequest(r.cluster, shouldCompare, r.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to create stream_route",
 		zap.String("id", obj.ID),
 		zap.Int32("server_port", obj.ServerPort),
@@ -162,7 +139,6 @@ func (r *streamRouteClient) Create(ctx context.Context, obj *v1.StreamRoute) (*v
 	url := r.url + "/" + obj.ID
 	log.Debugw("creating stream_route", zap.ByteString("body", data), zap.String("url", url))
 	resp, err := r.cluster.createResource(ctx, url, "streamRoute", data)
-	r.cluster.metricsCollector.IncrAPISIXRequest("streamRoute")
 	if err != nil {
 		log.Errorf("failed to create stream_route: %s", err)
 		return nil, err
@@ -176,6 +152,10 @@ func (r *streamRouteClient) Create(ctx context.Context, obj *v1.StreamRoute) (*v
 		log.Errorf("failed to reflect stream_route create to cache: %s", err)
 		return nil, err
 	}
+	if err := r.cluster.generatedObjCache.InsertStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect generated stream_route create to cache: %s", err)
+		return nil, err
+	}
 	return streamRoute, nil
 }
 
@@ -190,20 +170,28 @@ func (r *streamRouteClient) Delete(ctx context.Context, obj *v1.StreamRoute) err
 	}
 	url := r.url + "/" + obj.ID
 	if err := r.cluster.deleteResource(ctx, url, "streamRoute"); err != nil {
-		r.cluster.metricsCollector.IncrAPISIXRequest("streamRoute")
 		return err
 	}
-	r.cluster.metricsCollector.IncrAPISIXRequest("streamRoute")
 	if err := r.cluster.cache.DeleteStreamRoute(obj); err != nil {
 		log.Errorf("failed to reflect stream_route delete to cache: %s", err)
 		if err != cache.ErrNotFound {
 			return err
 		}
 	}
+	if err := r.cluster.generatedObjCache.DeleteStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect stream_route delete to generated cache: %s", err)
+		if err != cache.ErrNotFound {
+			return err
+		}
+	}
 	return nil
 }
 
-func (r *streamRouteClient) Update(ctx context.Context, obj *v1.StreamRoute) (*v1.StreamRoute, error) {
+func (r *streamRouteClient) Update(ctx context.Context, obj *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
+	if v, skip := skipRequest(r.cluster, shouldCompare, r.url, obj.ID, obj); skip {
+		return v, nil
+	}
+
 	log.Debugw("try to update stream_route",
 		zap.String("id", obj.ID),
 		zap.String("cluster", r.cluster.name),
@@ -218,7 +206,6 @@ func (r *streamRouteClient) Update(ctx context.Context, obj *v1.StreamRoute) (*v
 	}
 	url := r.url + "/" + obj.ID
 	resp, err := r.cluster.updateResource(ctx, url, "streamRoute", body)
-	r.cluster.metricsCollector.IncrAPISIXRequest("streamRoute")
 	if err != nil {
 		return nil, err
 	}
@@ -230,5 +217,107 @@ func (r *streamRouteClient) Update(ctx context.Context, obj *v1.StreamRoute) (*v
 		log.Errorf("failed to reflect stream_route update to cache: %s", err)
 		return nil, err
 	}
+	if err := r.cluster.generatedObjCache.InsertStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect generated stream_route update to cache: %s", err)
+		return nil, err
+	}
 	return streamRoute, nil
 }
+
+type streamRouteMem struct {
+	url string
+
+	resource string
+	cluster  *cluster
+}
+
+func newStreamRouteMem(c *cluster) StreamRoute {
+	return &streamRouteMem{
+		url:      c.baseURL + "/stream_routes",
+		resource: "stream_routes",
+		cluster:  c,
+	}
+}
+
+func (r *streamRouteMem) Get(ctx context.Context, name string) (*v1.StreamRoute, error) {
+	log.Debugw("try to look up route",
+		zap.String("name", name),
+		zap.String("cluster", r.cluster.name),
+	)
+	rid := id.GenID(name)
+	route, err := r.cluster.cache.GetStreamRoute(rid)
+	if err != nil {
+		log.Errorw("failed to find route in cache, will try to lookup from APISIX",
+			zap.String("name", name),
+			zap.Error(err),
+		)
+		return nil, err
+	}
+	return route, nil
+}
+
+// List is only used in cache warming up. So here just pass through
+// to APISIX.
+func (r *streamRouteMem) List(ctx context.Context) ([]*v1.StreamRoute, error) {
+	log.Debugw("try to list resource in APISIX",
+		zap.String("cluster", r.cluster.name),
+		zap.String("resource", r.resource),
+	)
+	streamRoutes, err := r.cluster.cache.ListStreamRoutes()
+	if err != nil {
+		log.Errorf("failed to list %s: %s", r.resource, err)
+		return nil, err
+	}
+	return streamRoutes, nil
+}
+
+func (r *streamRouteMem) Create(ctx context.Context, obj *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	if ok, err := r.cluster.validator.ValidateStreamPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.CreateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect stream_route create to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
+
+func (r *streamRouteMem) Delete(ctx context.Context, obj *v1.StreamRoute) error {
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return err
+	}
+	r.cluster.DeleteResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.DeleteStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect stream_route delete to cache: %s", err)
+		return err
+	}
+	return nil
+}
+
+func (r *streamRouteMem) Update(ctx context.Context, obj *v1.StreamRoute, shouldCompare bool) (*v1.StreamRoute, error) {
+	if shouldCompare && CompareResourceEqualFromCluster(r.cluster, obj.ID, obj) {
+		return obj, nil
+	}
+	if ok, err := r.cluster.validator.ValidateStreamPluginSchema(obj.Plugins); !ok {
+		return nil, err
+	}
+	data, err := json.Marshal(obj)
+	if err != nil {
+		return nil, err
+	}
+	r.cluster.UpdateResource(r.resource, obj.ID, data)
+	if err := r.cluster.cache.InsertStreamRoute(obj); err != nil {
+		log.Errorf("failed to reflect stream_route update to cache: %s", err)
+		return nil, err
+	}
+	return obj, nil
+}
