@@ -1,3 +1,4 @@
+//go:build tools
 // +build tools
 
 // Licensed to the Apache Software Foundation (ASF) under one or more
@@ -24,4 +25,5 @@ import (
 	_ "k8s.io/code-generator/cmd/deepcopy-gen"
 	_ "k8s.io/code-generator/cmd/informer-gen"
 	_ "k8s.io/code-generator/cmd/lister-gen"
+	_ "k8s.io/code-generator/cmd/register-gen"
 )
