@@ -16,6 +16,8 @@ package v1
 
 import (
 	"encoding/json"
+	"fmt"
+	"strings"
 
 	"github.com/incubator4/go-resty-expr/expr"
 )
@@ -86,13 +88,14 @@ type BasicAuthConsumerConfig struct {
 // used in Consumer object.
 // +k8s:deepcopy-gen=true
 type JwtAuthConsumerConfig struct {
-	Key          string `json:"key" yaml:"key"`
-	Secret       string `json:"secret,omitempty" yaml:"secret,omitempty"`
-	PublicKey    string `json:"public_key,omitempty" yaml:"public_key,omitempty"`
-	PrivateKey   string `json:"private_key" yaml:"private_key,omitempty"`
-	Algorithm    string `json:"algorithm,omitempty" yaml:"algorithm,omitempty"`
-	Exp          int64  `json:"exp,omitempty" yaml:"exp,omitempty"`
-	Base64Secret bool   `json:"base64_secret,omitempty" yaml:"base64_secret,omitempty"`
+	Key                 string `json:"key" yaml:"key"`
+	Secret              string `json:"secret,omitempty" yaml:"secret,omitempty"`
+	PublicKey           string `json:"public_key,omitempty" yaml:"public_key,omitempty"`
+	PrivateKey          string `json:"private_key" yaml:"private_key,omitempty"`
+	Algorithm           string `json:"algorithm,omitempty" yaml:"algorithm,omitempty"`
+	Exp                 int64  `json:"exp,omitempty" yaml:"exp,omitempty"`
+	Base64Secret        bool   `json:"base64_secret,omitempty" yaml:"base64_secret,omitempty"`
+	LifetimeGracePeriod int64  `json:"lifetime_grace_period,omitempty" yaml:"lifetime_grace_period,omitempty"`
 }
 
 // HMACAuthConsumerConfig is the rule config for hmac-auth plugin
@@ -110,6 +113,13 @@ type HMACAuthConsumerConfig struct {
 	MaxReqBody          int64    `json:"max_req_body,omitempty" yaml:"max_req_body,omitempty"`
 }
 
+// LDAPAuthConsumerConfig is the rule config for ldap-auth plugin
+// used in Consumer object.
+// +k8s:deepcopy-gen=true
+type LDAPAuthConsumerConfig struct {
+	UserDN string `json:"user_dn"`
+}
+
 // BasicAuthRouteConfig is the rule config for basic-auth plugin
 // used in Route object.
 // +k8s:deepcopy-gen=true
@@ -192,3 +202,81 @@ func (p *Headers) DeepCopy() *Headers {
 	p.DeepCopyInto(out)
 	return out
 }
+
+func (p *Headers) Add(headersToAdd []string) {
+	if p == nil {
+		return
+	}
+	if headersToAdd != nil {
+		addedHeader := make([]string, 0)
+		for _, h := range headersToAdd {
+			kv := strings.Split(h, ":")
+			if len(kv) < 2 {
+				continue
+			}
+			addedHeader = append(addedHeader, fmt.Sprintf("%s:%s", kv[0], kv[1]))
+		}
+		(*p)["add"] = addedHeader
+	}
+}
+
+func (p *Headers) GetAddedHeaders() []string {
+	if p == nil || (*p)["add"] == nil {
+		return nil
+	}
+	addedheaders, ok := (*p)["add"].([]string)
+	if ok {
+		return addedheaders
+	}
+	return nil
+}
+
+func (p *Headers) Set(headersToSet []string) {
+	if p == nil {
+		return
+	}
+	if headersToSet != nil {
+		setHeaders := make(map[string]string, 0)
+		for _, h := range headersToSet {
+			kv := strings.Split(h, ":")
+			if len(kv) < 2 {
+				continue
+			}
+			setHeaders[kv[0]] = kv[1]
+		}
+		(*p)["set"] = setHeaders
+	}
+}
+
+func (p *Headers) GetSetHeaders() map[string]string {
+	if p == nil || (*p)["set"] == nil {
+		return nil
+	}
+	addedheaders, ok := (*p)["set"].(map[string]string)
+	if ok {
+		return addedheaders
+	}
+	return nil
+}
+
+func (p *Headers) Remove(headersToRemove []string) {
+	if p == nil {
+		return
+	}
+	if headersToRemove != nil {
+		removeHeaders := make([]string, 0)
+		removeHeaders = append(removeHeaders, headersToRemove...)
+		(*p)["remove"] = removeHeaders
+	}
+}
+
+func (p *Headers) GetRemovedHeaders() []string {
+	if p == nil || (*p)["remove"] == nil {
+		return nil
+	}
+	removedHeaders, ok := (*p)["remove"].([]string)
+	if ok {
+		return removedHeaders
+	}
+	return nil
+}
