@@ -79,6 +79,13 @@ const (
 	// DefaultUpstreamTimeout represents the default connect,
 	// read and send timeout (in seconds) with upstreams.
 	DefaultUpstreamTimeout = 60
+
+	// PassHostPass represents pass option for pass_host Upstream settings.
+	PassHostPass = "pass"
+	// PassHostPass represents node option for pass_host Upstream settings.
+	PassHostNode = "node"
+	// PassHostPass represents rewrite option for pass_host Upstream settings.
+	PassHostRewrite = "rewrite"
 )
 
 var ValidSchemes map[string]struct{} = map[string]struct{}{
@@ -115,6 +122,7 @@ type Route struct {
 	UpstreamId      string           `json:"upstream_id,omitempty" yaml:"upstream_id,omitempty"`
 	Plugins         Plugins          `json:"plugins,omitempty" yaml:"plugins,omitempty"`
 	PluginConfigId  string           `json:"plugin_config_id,omitempty" yaml:"plugin_config_id,omitempty"`
+	FilterFunc      string           `json:"filter_func,omitempty" yaml:"filter_func,omitempty"`
 }
 
 // Vars represents the route match expressions of APISIX.
@@ -195,15 +203,17 @@ func (p *Plugins) DeepCopy() *Plugins {
 type Upstream struct {
 	Metadata `json:",inline" yaml:",inline"`
 
-	Type    string               `json:"type,omitempty" yaml:"type,omitempty"`
-	HashOn  string               `json:"hash_on,omitempty" yaml:"hash_on,omitempty"`
-	Key     string               `json:"key,omitempty" yaml:"key,omitempty"`
-	Checks  *UpstreamHealthCheck `json:"checks,omitempty" yaml:"checks,omitempty"`
-	Nodes   UpstreamNodes        `json:"nodes" yaml:"nodes"`
-	Scheme  string               `json:"scheme,omitempty" yaml:"scheme,omitempty"`
-	Retries *int                 `json:"retries,omitempty" yaml:"retries,omitempty"`
-	Timeout *UpstreamTimeout     `json:"timeout,omitempty" yaml:"timeout,omitempty"`
-	TLS     *ClientTLS           `json:"tls,omitempty" yaml:"tls,omitempty"`
+	Type         string               `json:"type,omitempty" yaml:"type,omitempty"`
+	HashOn       string               `json:"hash_on,omitempty" yaml:"hash_on,omitempty"`
+	Key          string               `json:"key,omitempty" yaml:"key,omitempty"`
+	Checks       *UpstreamHealthCheck `json:"checks,omitempty" yaml:"checks,omitempty"`
+	Nodes        UpstreamNodes        `json:"nodes" yaml:"nodes"`
+	Scheme       string               `json:"scheme,omitempty" yaml:"scheme,omitempty"`
+	Retries      *int                 `json:"retries,omitempty" yaml:"retries,omitempty"`
+	Timeout      *UpstreamTimeout     `json:"timeout,omitempty" yaml:"timeout,omitempty"`
+	TLS          *ClientTLS           `json:"tls,omitempty" yaml:"tls,omitempty"`
+	PassHost     string               `json:"pass_host,omitempty" yaml:"pass_host,omitempty"`
+	UpstreamHost string               `json:"upstream_host,omitempty" yaml:"upstream_host,omitempty"`
 
 	// for Service Discovery
 	ServiceName   string            `json:"service_name,omitempty" yaml:"service_name,omitempty"`
@@ -270,10 +280,12 @@ func (up Upstream) MarshalJSON() ([]byte, error) {
 			Key    string               `json:"key,omitempty" yaml:"key,omitempty"`
 			Checks *UpstreamHealthCheck `json:"checks,omitempty" yaml:"checks,omitempty"`
 			//Nodes   UpstreamNodes        `json:"nodes" yaml:"nodes"`
-			Scheme  string           `json:"scheme,omitempty" yaml:"scheme,omitempty"`
-			Retries *int             `json:"retries,omitempty" yaml:"retries,omitempty"`
-			Timeout *UpstreamTimeout `json:"timeout,omitempty" yaml:"timeout,omitempty"`
-			TLS     *ClientTLS       `json:"tls,omitempty" yaml:"tls,omitempty"`
+			Scheme       string           `json:"scheme,omitempty" yaml:"scheme,omitempty"`
+			Retries      *int             `json:"retries,omitempty" yaml:"retries,omitempty"`
+			Timeout      *UpstreamTimeout `json:"timeout,omitempty" yaml:"timeout,omitempty"`
+			HostPass     string           `json:"pass_host,omitempty" yaml:"pass_host,omitempty"`
+			UpstreamHost string           `json:"upstream_host,omitempty" yaml:"upstream_host,omitempty"`
+			TLS          *ClientTLS       `json:"tls,omitempty" yaml:"tls,omitempty"`
 
 			// for Service Discovery
 			ServiceName   string            `json:"service_name,omitempty" yaml:"service_name,omitempty"`
@@ -287,10 +299,12 @@ func (up Upstream) MarshalJSON() ([]byte, error) {
 			Key:    up.Key,
 			Checks: up.Checks,
 			//Nodes:   up.Nodes,
-			Scheme:  up.Scheme,
-			Retries: up.Retries,
-			Timeout: up.Timeout,
-			TLS:     up.TLS,
+			Scheme:       up.Scheme,
+			Retries:      up.Retries,
+			Timeout:      up.Timeout,
+			HostPass:     up.PassHost,
+			UpstreamHost: up.UpstreamHost,
+			TLS:          up.TLS,
 
 			ServiceName:   up.ServiceName,
 			DiscoveryType: up.DiscoveryType,
@@ -300,15 +314,17 @@ func (up Upstream) MarshalJSON() ([]byte, error) {
 		return json.Marshal(&struct {
 			Metadata `json:",inline" yaml:",inline"`
 
-			Type    string               `json:"type,omitempty" yaml:"type,omitempty"`
-			HashOn  string               `json:"hash_on,omitempty" yaml:"hash_on,omitempty"`
-			Key     string               `json:"key,omitempty" yaml:"key,omitempty"`
-			Checks  *UpstreamHealthCheck `json:"checks,omitempty" yaml:"checks,omitempty"`
-			Nodes   UpstreamNodes        `json:"nodes" yaml:"nodes"`
-			Scheme  string               `json:"scheme,omitempty" yaml:"scheme,omitempty"`
-			Retries *int                 `json:"retries,omitempty" yaml:"retries,omitempty"`
-			Timeout *UpstreamTimeout     `json:"timeout,omitempty" yaml:"timeout,omitempty"`
-			TLS     *ClientTLS           `json:"tls,omitempty" yaml:"tls,omitempty"`
+			Type         string               `json:"type,omitempty" yaml:"type,omitempty"`
+			HashOn       string               `json:"hash_on,omitempty" yaml:"hash_on,omitempty"`
+			Key          string               `json:"key,omitempty" yaml:"key,omitempty"`
+			Checks       *UpstreamHealthCheck `json:"checks,omitempty" yaml:"checks,omitempty"`
+			Nodes        UpstreamNodes        `json:"nodes" yaml:"nodes"`
+			Scheme       string               `json:"scheme,omitempty" yaml:"scheme,omitempty"`
+			Retries      *int                 `json:"retries,omitempty" yaml:"retries,omitempty"`
+			Timeout      *UpstreamTimeout     `json:"timeout,omitempty" yaml:"timeout,omitempty"`
+			HostPass     string               `json:"pass_host,omitempty" yaml:"pass_host,omitempty"`
+			UpstreamHost string               `json:"upstream_host,omitempty" yaml:"upstream_host,omitempty"`
+			TLS          *ClientTLS           `json:"tls,omitempty" yaml:"tls,omitempty"`
 
 			// for Service Discovery
 			//ServiceName   string            `json:"service_name,omitempty" yaml:"service_name,omitempty"`
@@ -317,15 +333,17 @@ func (up Upstream) MarshalJSON() ([]byte, error) {
 		}{
 			Metadata: up.Metadata,
 
-			Type:    up.Type,
-			HashOn:  up.HashOn,
-			Key:     up.Key,
-			Checks:  up.Checks,
-			Nodes:   up.Nodes,
-			Scheme:  up.Scheme,
-			Retries: up.Retries,
-			Timeout: up.Timeout,
-			TLS:     up.TLS,
+			Type:         up.Type,
+			HashOn:       up.HashOn,
+			Key:          up.Key,
+			Checks:       up.Checks,
+			Nodes:        up.Nodes,
+			Scheme:       up.Scheme,
+			Retries:      up.Retries,
+			Timeout:      up.Timeout,
+			HostPass:     up.PassHost,
+			UpstreamHost: up.UpstreamHost,
+			TLS:          up.TLS,
 
 			//ServiceName:   up.ServiceName,
 			//DiscoveryType: up.DiscoveryType,
@@ -473,8 +491,8 @@ type StreamRoute struct {
 // GlobalRule represents the global_rule object in APISIX.
 // +k8s:deepcopy-gen=true
 type GlobalRule struct {
-	ID      string  `json:"id,omitempty" yaml:"id,omitempty"`
-	Plugins Plugins `json:"plugins,omitempty" yaml:"plugins,omitempty"`
+	ID      string  `json:"id" yaml:"id"`
+	Plugins Plugins `json:"plugins" yaml:"plugins"`
 }
 
 // Consumer represents the consumer object in APISIX.
@@ -566,6 +584,13 @@ func NewDefaultPluginConfig() *PluginConfig {
 	}
 }
 
+// NewDefaultGlobalRule returns an empty PluginConfig with default values.
+func NewDefaultGlobalRule() *GlobalRule {
+	return &GlobalRule{
+		Plugins: make(Plugins),
+	}
+}
+
 // ComposeUpstreamName uses namespace, name, subset (optional), port, resolveGranularity info to compose
 // the upstream name.
 // the resolveGranularity is not composited in the upstream name when it is endpoint.
@@ -656,7 +681,7 @@ func ComposeConsumerName(namespace, name string) string {
 }
 
 // ComposePluginConfigName uses namespace, name to compose
-// the route name.
+// the plugin_config name.
 func ComposePluginConfigName(namespace, name string) string {
 	// FIXME Use sync.Pool to reuse this buffer if the upstream
 	// name composing code path is hot.
@@ -670,6 +695,21 @@ func ComposePluginConfigName(namespace, name string) string {
 	return buf.String()
 }
 
+// ComposeGlobalRuleName uses namespace, name to compose
+// the global_rule name.
+func ComposeGlobalRuleName(namespace, name string) string {
+	// FIXME Use sync.Pool to reuse this buffer if the upstream
+	// name composing code path is hot.
+	p := make([]byte, 0, len(namespace)+len(name)+1)
+	buf := bytes.NewBuffer(p)
+
+	buf.WriteString(namespace)
+	buf.WriteByte('_')
+	buf.WriteString(name)
+
+	return buf.String()
+}
+
 // Schema represents the schema of APISIX objects.
 type Schema struct {
 	Name    string `json:"name,omitempty" yaml:"name,omitempty"`
